"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Shell = void 0;
var computedFrom_1 = require("./computedFrom");
var Dict_1 = require("./polyfill/Dict");
var WebSocket_1 = require("./polyfill/WebSocket");
var Shell = /** @class */ (function () {
    function Shell(ws) {
        /** The connection is not yet open. */
        this.CONNECTING = WebSocket_1.default.CONNECTING;
        /**  The connection is open and ready to communicate. */
        this.OPEN = WebSocket_1.default.OPEN;
        /**  The connection is in the process of closing. */
        this.CLOSING = WebSocket_1.default.CLOSING;
        /**  The connection is closed or couldn't be opened. */
        this.CLOSED = WebSocket_1.default.CLOSED;
        this.closing = false;
        this._readyState = WebSocket_1.default.CONNECTING;
        this.listeners = new Dict_1.Dict();
        if (!this.dispatchEvent) {
            throw new TypeError("Failed to construct. Please use the 'new' operator");
        }
        this.forwardListener = this.dispatchEvent.bind(this);
        if (ws) {
            this.ws = ws;
            this.forwardEvents();
        }
    }
    Object.defineProperty(Shell.prototype, "onclose", {
        /**
         * An event listener to be called when the WebSocket connection's **readyState** changes to **CLOSED**.
         * The listener receives a _CloseEvent_ named "close".
         */
        get: function () {
            return this._onclose;
        },
        set: function (f) {
            this._onclose = f;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "onerror", {
        /**
         * An event listener to be called when an error occurs. This is a simple event named "error".
         */
        get: function () {
            return this._onerror;
        },
        set: function (f) {
            this._onerror = f;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "onmessage", {
        /**
         * An event listener to be called when a message is received from the server.
         * The listener receives a _MessageEvent_ named "message".
         */
        get: function () {
            return this.getOnmessage();
        },
        set: function (f) {
            this.setOnmessage(f);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "onopen", {
        /**
         * An event listener to be called when the WebSocket connection's readyState changes to **OPEN**;
         * this indicates that the connection is ready to send and receive data.
         * The event is a simple one with the name "open".
         */
        get: function () {
            return this._onopen;
        },
        set: function (f) {
            this._onopen = f;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "readyState", {
        /**
         * The current state of the connection; this is one of the Ready state constants. **Read only.**
         */
        get: function () {
            return this.getReadyState();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "url", {
        /**
         * The URL as resolved by the constructor. This is always an absolute URL. **Read only.**
         */
        get: function () {
            return this.ws.url;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "bufferedAmount", {
        /**
         * The number of bytes of data that have been queued using calls to _send()_ but not yet transmitted to the network.
         * This value resets to zero once all queued data has been sent. This value does not reset to zero when the
         * connection is closed; if you keep calling _send()_, this will continue to climb. **Read only**
         */
        get: function () {
            return this.ws.bufferedAmount;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "extensions", {
        /**
         * The extensions selected by the server. This is currently only the empty string or a list
         * of extensions as negotiated by the connection.
         */
        get: function () {
            return this.ws.extensions;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Shell.prototype, "protocol", {
        get: function () {
            return this.ws.protocol;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Closes the WebSocket connection or connection attempt, if any.
     * If the connection is already CLOSED, this method does nothing.
     * @param {number} A numeric value indicating the status code explaining why the connection is being closed.
     * If this parameter is not specified, a default value of 1005 is assumed.
     * See the list of status codes https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent#Status_codes
     * on the CloseEvent page for permitted values.
     * @param {string} A human-readable string explaining why the connection is closing.
     * This string must be no longer than 123 bytes of UTF-8 text (not characters).
     */
    Shell.prototype.close = function (code, reason) {
        if (code === void 0) { code = 1000; }
        this.ws.close(code, reason);
    };
    /**
     * Enqueues the specified data to be transmitted to the server over the WebSocket connection,
     * increasing the value of bufferedAmount by the number of bytes needed to contain the data.
     * If the data can't be sent (for example, because it needs to be buffered but the buffer is full), t
     * he socket is closed automatically.
     * @param data The data to send to the server. It may be one of the following types:
     * - **USVString**
     * A text string. The string is added to the buffer in UTF-8 format, and the value of bufferedAmount is increased
     * by the number of bytes required to represent the UTF-8 string.
     * - **ArrayBuffer**
     * You can send the underlying binary data used by a typed array object; its binary data contents are queued in the
     * buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     * - **Blob**
     * Specifying a Blob enqueues the blob's raw data to be transmitted in a binary frame. The value of bufferedAmount
     * is increased by the byte size of that raw data.
     * - **ArrayBufferView**
     * You can send any JavaScript typed array object as a binary frame; its binary data contents are queued in the
     * buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     */
    Shell.prototype.send = function (data) {
        if (this.ws.readyState !== WebSocket_1.default.OPEN) {
            var err = new Error("WebSocket is not open ws is " + this.ws.readyState + ", local is " + this.readyState);
            throw err;
        }
        this.ws.send(data);
    };
    /**
     * Register an event handler of a specific event type on the EventTarget.
     * @param {K} A case-sensitive string representing the event type to listen for.
     * @param {(ev: IHydratedWebSocketEventMap[K]) => any} The object which receives a notification
     * (an object that implements the Event interface) when an event of the specified type occurs.
     * This must be an object implementing the EventListener interface, or a JavaScript function.
     * @param {boolean} A Boolean indicating whether events of this type will be dispatched to the registered
     * listener before being dispatched to any EventTarget beneath it in the DOM tree.
     * Events that are bubbling upward through the tree will not trigger a listener designated to use capture.
     * Event bubbling and capturing are two ways of propagating events which occur in an element that is nested
     * within another element, when both elements have registered a handle for that event.
     * The event propagation mode determines the order in which elements receive the event.
     * See DOM Level 3 Events and JavaScript Event order for a detailed explanation.
     * If not specified, useCapture defaults to false.
     */
    Shell.prototype.addEventListener = function (type, listener, useCapture) {
        var listeners = this.listeners.get(type);
        if (!listeners) {
            listeners = [];
            this.listeners.set(type, listeners);
        }
        listeners.push({ listener: listener, useCapture: useCapture });
    };
    /**
     * Removes an event listener from the EventTarget.
     * @param {K} A string which specifies the type of event for which to remove an event listener.
     * @param {(ev: IHydratedWebSocketEventMap[K]) => any} The EventListener
     * function of the event handler to remove from the event target.
     * @param {boolean} Specifies whether the EventListener to be removed is registered as a capturing
     * listener or not. If this parameter is absent, a default value of false is assumed.
     */
    Shell.prototype.removeEventListener = function (type, listener, useCapture) {
        var listeners = this.listeners.get(type);
        if (listeners) {
            this.listeners.set(type, listeners.filter(function (l) { return l.listener !== listener || l.useCapture !== useCapture; }));
        }
    };
    /**
     * Dispatch an event to this EventTarget.
     * @param {} The Event object to be dispatched.
     * @returns {boolean} The return value is **false** if event is cancelable and at least one of the event
     * handlers which handled this event called Event.preventDefault(). Otherwise it returns true.
     */
    Shell.prototype.dispatchEvent = function (evt) {
        var _this = this;
        var method = this["_on" + evt.type];
        if (typeof method === "function") {
            method.call(this, evt);
        }
        return (this.listeners.get(evt.type) || [])
            .some(function (_a) {
            var listener = _a.listener;
            return listener.call(_this, evt) === false;
        }) === void 0;
    };
    Shell.prototype.getOnmessage = function () {
        return this._onmessage;
    };
    Shell.prototype.setOnmessage = function (f) {
        this._onmessage = f;
    };
    Shell.prototype.forwardEvents = function (list) {
        var _this = this;
        (list || ["close", "message", "open"]).forEach(function (event) {
            return _this.ws.addEventListener(event, _this.forwardListener);
        });
    };
    Shell.prototype.stopForwardingEvents = function () {
        this.ws.removeEventListener("close", this.forwardListener);
        this.ws.removeEventListener("message", this.forwardListener);
        this.ws.removeEventListener("open", this.forwardListener);
    };
    Shell.prototype.getReadyState = function () {
        return this.ws.readyState;
    };
    __decorate([
        computedFrom_1.computedFrom("_onclose")
    ], Shell.prototype, "onclose", null);
    __decorate([
        computedFrom_1.computedFrom("_onerror")
    ], Shell.prototype, "onerror", null);
    __decorate([
        computedFrom_1.computedFrom("_onmessage")
    ], Shell.prototype, "onmessage", null);
    __decorate([
        computedFrom_1.computedFrom("_onopen")
    ], Shell.prototype, "onopen", null);
    __decorate([
        computedFrom_1.computedFrom("ws.readyState")
    ], Shell.prototype, "readyState", null);
    __decorate([
        computedFrom_1.computedFrom("ws.url")
    ], Shell.prototype, "url", null);
    __decorate([
        computedFrom_1.computedFrom("ws.bufferedAmount")
    ], Shell.prototype, "bufferedAmount", null);
    __decorate([
        computedFrom_1.computedFrom("ws.extensions")
    ], Shell.prototype, "extensions", null);
    __decorate([
        computedFrom_1.computedFrom("ws.protocol")
    ], Shell.prototype, "protocol", null);
    return Shell;
}());
exports.Shell = Shell;
//# sourceMappingURL=Shell.js.map