"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cable = void 0;
var Dict_1 = require("../polyfill/Dict");
var ErrorEvent_1 = require("../polyfill/ErrorEvent");
var Shell_1 = require("../Shell");
var CableError_1 = require("./CableError");
var uuid_1 = require("./uuid");
var isVoid = function (v) { return v === void 0; };
var voidNull = function (v) { return v === null ? void 0 : v; };
/**
 * RPC over websocket, using the JSON-RPC 2.0 Specification
 * http://www.jsonrpc.org/specification
 *
 * A `Cable` establish an RPC communication over a websocket, there is no server/client role, the same cable can
 * you can define methods on both sides of the cable and call the remote methods
 *
 * @example
 * ```typescript
 *
 * // Client 1
 * const cable = new Cable(ws);
 * cable.register("ping", async () => {
 *   return "pong";
 * });
 * cable.notify("hello", {name:"client 1"});
 *
 * // Client 2
 * const cable = new Cable(ws);
 * cable.register("hello", async ({name:string}) => {
 *   console.log(`${name} said hello`);
 *  });
 * try {
 *   const res = await cable.request("ping");
 *   assert.equal(res,"pong");
 * } catch(e) {
 *   if(e.code === Cable.SERVER_ERROR) {
 *     console.log("Implementation error on the server");
 *   }
 *   throw e;
 * }
 * ```
 */
var Cable = /** @class */ (function (_super) {
    __extends(Cable, _super);
    /**
     * @param ws  An object compatible with the WebSocket interface.
     */
    function Cable(ws) {
        var _this = _super.call(this) || this;
        _this.calls = new Dict_1.Dict();
        _this.methods = new Dict_1.Dict();
        _this.receivedMessage = function (e) {
            var data = _this.parseMessage(e.data);
            if (!data) {
                return;
            }
            if (!isVoid(data.error)) {
                _this.rpcError(data.id, data.error.code, data.error.message);
            }
            else if (!isVoid(data.result)) {
                _this.rpcResult(data.id, data.result);
            }
            else if (!isVoid(data.method)) {
                _this.rpcCall(data.id, data.method, data.params);
            }
            else {
                _this.sendError(data.id, Cable.INVALID_PARAMS, "Unknown message type");
            }
        };
        _this.ws = ws;
        _this.forwardEvents(["close", "open"]);
        _this.ws.addEventListener("message", _this.receivedMessage);
        return _this;
    }
    /**
     * Register a new method on the websocket
     * @param {string} method name
     * @param {(params: any) => Promise<any>} method handler
     */
    Cable.prototype.register = function (name, method) {
        this.methods.set(name, method);
    };
    /**
     * Make a Rpc call
     * @param {string} method name
     * @param {object | any[]} params
     * @param {number} if set will reject if no response is received after `timeout` ms
     * @returns {Promise<any>}
     */
    Cable.prototype.request = function (method, params, timeout) {
        return __awaiter(this, void 0, void 0, function () {
            var id, p;
            var _this = this;
            return __generator(this, function (_a) {
                this.guardParameters(params);
                Cable.index++;
                id = Cable.id + "-" + Cable.index;
                p = new Promise(function (resolve, reject) { return _this.calls.set(id, {
                    reject: reject,
                    resolve: resolve,
                    timeout: timeout ? setTimeout(function () { return _this.timeout(id); }, timeout) : null,
                }); });
                this.sendMessage(id, { method: method, params: params });
                return [2 /*return*/, p];
            });
        });
    };
    /**
     * Unlike the _request_, notify will not wait for the server to reply
     * @param {string} method name
     * @param {object | any[]} params
     * @returns {Promise<any>}
     */
    Cable.prototype.notify = function (method, params) {
        this.guardParameters(params);
        this.sendMessage(null, { method: method, params: params });
    };
    /**
     * Destroy the Cable, send a timeout for all pending calls and clean the registered methods
     */
    Cable.prototype.destroy = function () {
        var _this = this;
        this.stopForwardingEvents();
        this.ws.removeEventListener("message", this.receivedMessage);
        this.calls.keys().forEach(function (id) { return _this.timeout(id); });
        this.calls.clear();
        this.methods.clear();
    };
    Cable.prototype.guardParameters = function (params) {
        if (params === null || (!isVoid(params) && typeof params !== "object")) {
            throw new CableError_1.CableError("params accept an array or an object, provided a " + (params === null ? null : typeof params), Cable.INVALID_CLIENT_PARAMS);
        }
    };
    Cable.prototype.timeout = function (id) {
        if (this.calls.has(id)) {
            this.calls.get(id).reject(new CableError_1.CableError("Request timeout", Cable.TIMEOUT_ERROR));
            this.calls.delete(id);
        }
    };
    Cable.prototype.sendMessage = function (id, message) {
        if (isVoid(id)) {
            return;
        }
        message.jsonrpc = "2.0";
        if (id !== null) {
            message.id = id;
        }
        try {
            this.ws.send(JSON.stringify(message));
        }
        catch (e) {
            // ignore
        }
    };
    Cable.prototype.sendError = function (id, code, messageOrError) {
        var message = messageOrError ? messageOrError.toString() : "Unknown error";
        if (messageOrError instanceof CableError_1.CableError) {
            code = messageOrError.code;
            message = messageOrError.message;
        }
        this.sendMessage(id, { error: { code: code, message: message } });
    };
    Cable.prototype.parseMessage = function (message) {
        try {
            var data = JSON.parse(message);
            data.id = voidNull(data.id);
            return data;
        }
        catch (e) {
            this.sendError(null, Cable.PARSE_ERROR, e);
            return;
        }
    };
    Cable.prototype.rpcCall = function (id, method, params) {
        var _this = this;
        if (this.methods.has(method)) {
            this.methods.get(method).call(this, params)
                .then(function (res) {
                _this.sendMessage(id, { result: res || null });
            })
                .catch(this.sendError.bind(this, id, Cable.SERVER_ERROR));
        }
        else {
            this.sendError(id, Cable.METHOD_NOT_FOUND, "Method '" + method + "' not found");
        }
    };
    Cable.prototype.rpcResult = function (id, results) {
        if (this.calls.has(id)) {
            clearTimeout(this.calls.get(id).timeout);
            this.calls.get(id).resolve(results);
            this.calls.delete(id);
        }
        else {
            this.dispatchEvent(new ErrorEvent_1.default("error", {
                error: new CableError_1.CableError("Response received for an unknown request", Cable.UNKNOWN_REQUEST),
            }));
        }
    };
    Cable.prototype.rpcError = function (id, code, message) {
        var error = new CableError_1.CableError(message, code);
        if (this.calls.has(id)) {
            clearTimeout(this.calls.get(id).timeout);
            this.calls.get(id).reject(error);
            this.calls.delete(id);
        }
        else {
            this.dispatchEvent(new ErrorEvent_1.default("error", { error: error }));
        }
    };
    /**
     *  Invalid JSON was received by the server.
     *  An error occurred on the server while parsing the JSON text.
     */
    Cable.PARSE_ERROR = -32700;
    /**
     * The JSON sent is not a valid Request object.
     */
    Cable.INVALID_REQUEST = -32600;
    /**
     *  The method does not exist / is not available.
     */
    Cable.METHOD_NOT_FOUND = -32601;
    /**
     *  Invalid method parameter(s).
     */
    Cable.INVALID_PARAMS = -32602;
    /**
     *  Internal JSON-RPC error.
     */
    Cable.INTERNAL_ERROR = -32603;
    /**
     * Generic server-errors
     */
    Cable.SERVER_ERROR = -32000;
    /**
     * Timeout server-errors
     */
    Cable.TIMEOUT_ERROR = -32001;
    /**
     * Response received for an unknown request server-errors
     */
    Cable.UNKNOWN_REQUEST = -32002;
    /**
     *  Invalid method parameter(s) on the client
     */
    Cable.INVALID_CLIENT_PARAMS = -32603;
    Cable.id = uuid_1.uuid();
    Cable.index = 0;
    return Cable;
}(Shell_1.Shell));
exports.Cable = Cable;
//# sourceMappingURL=Cable.js.map