"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Router = void 0;
var Dict_1 = require("../polyfill/Dict");
var WebSocket_1 = require("../polyfill/WebSocket");
var RoutedWebSocket_1 = require("./RoutedWebSocket");
/**
 * ALPHA, do not use
 * The Router route messages from different servers
 */
var Router = /** @class */ (function () {
    function Router() {
        this.localWebSockets = new Dict_1.Dict();
        this.virtualWebSockets = new Dict_1.Dict();
    }
    Router.prototype.clear = function () {
        var _this = this;
        this.localWebSockets.keys().forEach(function (key) { return _this.delete(key); });
        this.virtualWebSockets.keys().forEach(function (key) { return _this.delete(key); });
    };
    Router.prototype.destroy = function () {
        this.localWebSockets.values().forEach(function (ws) { return ws.close(); });
        this.virtualWebSockets.values().forEach(function (ws) { return ws.close(); });
        this.clear();
    };
    Object.defineProperty(Router.prototype, "connector", {
        set: function (value) {
            if (this._connector) {
                this._connector.onBroadcast = void 0;
                this._connector.onMessage = void 0;
                this._connector.onClose = void 0;
                this._connector.onStatus = void 0;
            }
            this._connector = value;
            if (value) {
                this._connector.onBroadcast = this.onBroadcast.bind(this);
                this._connector.onMessage = this.onMessage.bind(this);
                this._connector.onClose = this.onClose.bind(this);
                this._connector.onStatus = this.onStatus.bind(this);
                this._connector.onRequestReadyState = this.onRequestReadyState.bind(this);
            }
        },
        enumerable: false,
        configurable: true
    });
    Router.prototype.onMessage = function (id, data) {
        var ws = this.localWebSockets.get(id);
        if (ws && ws.readyState === WebSocket_1.default.OPEN) {
            ws.send(data);
            return true;
        }
    };
    Router.prototype.onRequestReadyState = function (id, data) {
        var ws = this.localWebSockets.get(id);
        if (ws && ws.readyState === WebSocket_1.default.OPEN) {
            return ws.readyState;
        }
    };
    Router.prototype.onStatus = function (id, status) {
        if (status === WebSocket_1.default.OPEN && this.localWebSockets.has(id) &&
            this.localWebSockets.get(id).readyState !== status) {
            if (this.virtualWebSockets.has(id)) {
                this.virtualWebSockets.get(id).setReadyState(WebSocket_1.default.CLOSED);
            }
            this.localWebSockets.delete(id);
        }
        var ws = this.virtualWebSockets.get(id);
        if (ws) {
            ws.setReadyState(status);
        }
    };
    Router.prototype.onBroadcast = function (data) {
        this.localWebSockets.values().forEach(function (ws) {
            if (ws.readyState === WebSocket_1.default.OPEN) {
                ws.send(data);
            }
        });
    };
    Router.prototype.onClose = function (id, code, reason) {
        var ws = this.localWebSockets.get(id);
        if (ws) {
            ws.close(code, reason);
            return true;
        }
    };
    Router.prototype.emitState = function (id, ws) {
        this.setReadyState(id, ws.readyState);
        if (this._connector) {
            this._connector.readyState(id, ws.readyState);
        }
    };
    Router.prototype.set = function (id, ws) {
        var _this = this;
        if (this.localWebSockets.get(id) === ws) {
            return;
        }
        this.close(id, 1000, "Duplicate websocket");
        var openListener = function () { return _this.emitState(id, ws); };
        var messageListener = function (event) { return _this.emitMessage(id, event); };
        var closeListener = function (event) {
            if (_this.localWebSockets.get(id) === ws) {
                _this.emitState(id, ws);
            }
        };
        ws.addEventListener("open", openListener);
        ws.addEventListener("message", messageListener);
        ws.addEventListener("close", closeListener);
        if (this.virtualWebSockets.has(id) && this.virtualWebSockets.get(id).readyState === ws.readyState) {
            this.virtualWebSockets.get(id).setReadyState(WebSocket_1.default.CLOSED);
        }
        this.localWebSockets.set(id, ws);
        this.emitState(id, ws);
    };
    Router.prototype.delete = function (id) {
        this.localWebSockets.delete(id);
        this.setReadyState(id, WebSocket_1.default.CLOSED);
    };
    Router.prototype.get = function (id) {
        if (!this.virtualWebSockets.has(id)) {
            var routedWs_1 = this.createRoutedWebsocket(id);
            this.virtualWebSockets.set(id, routedWs_1);
            if (this.localWebSockets.has(id)) {
                routedWs_1.setReadyState(this.localWebSockets.get(id).readyState);
            }
            else {
                if (this._connector) {
                    this._connector.requestReadyState(id)
                        .then(function (readyState) {
                        if (readyState && !routedWs_1.readyState) {
                            routedWs_1.setReadyState(readyState);
                        }
                    })
                        .catch(function () { return void 0; });
                }
            }
        }
        var ws = this.virtualWebSockets.get(id);
        return ws;
    };
    Router.prototype.onMessageSubscribe = function (id, ws) {
        if (this._connector) {
            this._connector.subscribe(id, ws);
        }
    };
    Router.prototype.onMessageUnsubscribe = function (id, ws) {
        if (this._connector) {
            this._connector.unsubscribe(id, ws);
        }
    };
    Router.prototype.broadcast = function (data) {
        this.onBroadcast(data);
        if (this._connector) {
            this._connector.broadcast(data);
        }
    };
    Router.prototype.createRoutedWebsocket = function (id) {
        var _this = this;
        return new RoutedWebSocket_1.RoutedWebSocket(function (data) { return _this.send(id, data); }, function (code, reason) { return _this.close(id, code, reason); }, function (vWs) { return _this.onMessageSubscribe(id, vWs); }, function (vWs) { return _this.onMessageUnsubscribe(id, vWs); });
    };
    Router.prototype.send = function (id, data) {
        var ws = this.localWebSockets.get(id);
        if (ws && ws.readyState === WebSocket_1.default.OPEN) {
            ws.send(data);
        }
        else if (this._connector) {
            this._connector.send(id, data);
        }
    };
    Router.prototype.setReadyState = function (id, state) {
        if (this.virtualWebSockets.has(id)) {
            this.virtualWebSockets.get(id).setReadyState(state);
        }
    };
    Router.prototype.emitMessage = function (id, event) {
        if (this.localWebSockets.has(id) && this.virtualWebSockets.has(id)) {
            this.virtualWebSockets.get(id).emitMessage(event);
        }
        if (this._connector) {
            this._connector.emitMessage(id, event);
        }
    };
    Router.prototype.close = function (id, code, reason) {
        this.setReadyState(id, WebSocket_1.default.CLOSED);
        if (this.localWebSockets.has(id)) {
            this.localWebSockets.get(id).close(code, reason);
        }
        else if (this._connector) {
            this._connector.close(id, code, reason);
        }
    };
    return Router;
}());
exports.Router = Router;
//# sourceMappingURL=Router.js.map