"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tank = void 0;
var Shell_1 = require("../Shell");
/**
 * The Tank allow you to send message without checking the state of the connection, the messages will be delayed until
 * the connection is open.
 */
var Tank = /** @class */ (function (_super) {
    __extends(Tank, _super);
    function Tank(ws) {
        var _this = _super.call(this) || this;
        _this.buffer = [];
        _this.ws = ws;
        _this.forwardEvents();
        _this.addEventListener("open", function () { return _this.flush(); });
        return _this;
        // @todo
        // The tank do not send twice an open event so the readyState can
        // change without an open or close event
        // detect this is a tank, add a readyState change event
    }
    /**
     * Closes the WebSocket connection or connection attempt, if any.
     * If the connection is already CLOSED, this method does nothing.
     * @param {number} A numeric value indicating the status code explaining why the connection is being closed.
     * If this parameter is not specified, a default value of 1005 is assumed.
     * See the list of status codes https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent#Status_codes
     * on the CloseEvent page for permitted values.
     * @param {string} A human-readable string explaining why the connection is closing.
     * This string must be no longer than 123 bytes of UTF-8 text (not characters).
     */
    Tank.prototype.close = function (code, reason) {
        if (code === void 0) { code = 1000; }
        this.stopFlushInterval();
        _super.prototype.close.call(this, code, reason);
    };
    /**
     * Unlike the regular _send_ command of a WebSocket the messages are stored until the connection is **OPEN** and
     * will
     * be flushed automatically
     * @param data The data to send to the server. It may be one of the following types:
     * - **USVString**
     * A text string. The string is added to the buffer in UTF-8 format, and the value of bufferedAmount is increased
     * by the number of bytes required to represent the UTF-8 string.
     * - **ArrayBuffer**
     * You can send the underlying binary data used by a typed array object; its binary data contents are queued in
     * the buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     * - **Blob**
     * Specifying a Blob enqueues the blob's raw data to be transmitted in a binary frame. The value of bufferedAmount
     * is increased by the byte size of that raw data.
     * - **ArrayBufferView**
     * You can send any JavaScript typed array object as a binary frame; its binary data contents are queued in the
     * buffer, increasing the value of bufferedAmount by the requisite number of bytes.
     */
    Tank.prototype.send = function (data) {
        if (!data) {
            return;
        }
        this.buffer.unshift(data);
        this.flush();
    };
    Tank.prototype.startFlushInterval = function () {
        var _this = this;
        if (!this.flushInterval) {
            this.flushInterval = setInterval(function () { return _this.flush(); }, 100);
        }
    };
    Tank.prototype.stopFlushInterval = function () {
        if (this.flushInterval) {
            clearInterval(this.flushInterval);
        }
        this.flushInterval = null;
    };
    Tank.prototype.flush = function () {
        if (this.ws.readyState === this.OPEN) {
            var data = void 0;
            while (data = this.buffer.pop()) { // tslint:disable-line:no-conditional-assignment
                this.ws.send(data);
            }
            this.stopFlushInterval();
        }
        else {
            this.startFlushInterval();
        }
    };
    return Tank;
}(Shell_1.Shell));
exports.Tank = Tank;
//# sourceMappingURL=Tank.js.map