"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exponentialTruncatedBackoff = void 0;
/**
 * Truncated exponential backoff default policy for the Waterfall retry
 * see https://en.wikipedia.org/wiki/Exponential_backoff
 * @param {number} slotTime
 * @param {number} truncate
 * @returns {(attempt: number, websocket: WebSocket) => number}
 */
exports.exponentialTruncatedBackoff = function (slotTime, truncate) {
    if (slotTime === void 0) { slotTime = 100; }
    if (truncate === void 0) { truncate = Number.MAX_VALUE; }
    return function (attempt, websocket) {
        return Math.random() * slotTime * (Math.pow(2, Math.min(attempt, truncate)) - 1);
    };
};
//# sourceMappingURL=exponentialTruncatedBackoff.js.map