"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
var __1 = require("..");
var Dict_1 = require("../polyfill/Dict");
var ErrorEvent_1 = require("../polyfill/ErrorEvent");
var WebSocket_1 = require("../polyfill/WebSocket");
var Client = /** @class */ (function () {
    function Client(uuid, ws, onConnection) {
        var _this = this;
        this.uuid = uuid;
        this.onConnection = onConnection;
        this.webSockets = new Dict_1.Dict();
        this.identified = false;
        this.identifyRunning = false;
        var cablePipe = new __1.Pipe(ws, "WOHC");
        this.dataPipe = new __1.Pipe(ws, "WOHD");
        this.cable = new __1.Cable(new __1.Tank(cablePipe));
        this.registerCableMethods();
        if (ws.readyState === WebSocket_1.default.OPEN) {
            this.identify();
        }
        ws.addEventListener("open", function () {
            _this.identified = false;
            _this.identifyRunning = false;
            _this.identify();
        });
    }
    Client.prototype.connect = function (uuid) {
        var _this = this;
        var channel = (Math.random().toString().replace(".", "") +
            Math.random().toString().replace(".", "")).slice(0, 32);
        var pipe = new __1.Pipe(this.dataPipe, channel, 32);
        this.cable.request("connect", { uuid: uuid, channel: channel }).catch(function (e) { return pipe.dispatchEvent(new ErrorEvent_1.default(e)); });
        pipe.addEventListener("close", function () {
            try {
                _this.cable.notify("close", { channel: channel });
            }
            catch (e) {
                // ignore
            }
        });
        return pipe;
    };
    Client.prototype.registerCableMethods = function () {
        var _this = this;
        this.cable.register("open", function (_a) {
            var channel = _a.channel;
            if (_this.webSockets.has(channel)) {
                return new Promise(function (resolve) { return setTimeout(resolve, 0); });
            }
            var pipe = new __1.Pipe(_this.dataPipe, channel, 32);
            _this.webSockets.set(channel, pipe);
            _this.onConnection(pipe);
            return new Promise(function (resolve) { return setTimeout(resolve, 0); });
        });
        this.cable.register("close", function (_a) {
            var channel = _a.channel;
            if (_this.webSockets.has(channel)) {
                _this.webSockets.get(channel).close();
                _this.webSockets.delete(channel);
            }
            return new Promise(function (resolve) { return setTimeout(resolve, 0); });
        });
    };
    Client.prototype.identify = function () {
        var _this = this;
        if (this.identified || this.identifyRunning) {
            return;
        }
        this.identifyRunning = true;
        this.cable.request("identity", { uuid: this.uuid })
            .then(function () {
            _this.identifyRunning = false;
            _this.identified = true;
        })
            .catch(function () {
            _this.identifyRunning = false;
            setTimeout(function () {
                _this.identify();
            }, 100);
        });
    };
    return Client;
}());
exports.Client = Client;
//# sourceMappingURL=Client.js.map