"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
var Cable_1 = require("../cable/Cable");
var Pipe_1 = require("../pipe/Pipe");
var Dict_1 = require("../polyfill/Dict");
var WebSocket_1 = require("../polyfill/WebSocket");
var Router_1 = require("../router/Router");
var Server = /** @class */ (function () {
    function Server() {
        this.clients = new Dict_1.Dict();
        this.buffers = new Dict_1.Dict();
        this.channels = new Dict_1.Dict();
        this.router = new Router_1.Router();
    }
    Server.prototype.destroy = function () {
        this.router.destroy();
        this.channels.values()
            .forEach(function (_a) {
            var source = _a.source, target = _a.target;
            if (source) {
                source.close();
            }
            if (target) {
                target.close();
            }
        });
        this.clients.values()
            .forEach(function (_a) {
            var data = _a.data, cable = _a.cable;
            if (data) {
                data.close();
            }
            if (cable) {
                cable.close();
            }
        });
        this.clients = void 0;
        this.buffers = void 0;
        this.channels = void 0;
    };
    Server.prototype.addWebSocket = function (ws) {
        var _this = this;
        var cablePipe = new Pipe_1.Pipe(ws, "WOHC");
        var cable = new Cable_1.Cable(cablePipe);
        var dataPipe = new Pipe_1.Pipe(ws, "WOHD");
        cable.register("identity", function (_a) {
            var uuid = _a.uuid;
            _this.router.set(uuid, ws);
            return new Promise(function (resolve) { return setTimeout(resolve, 0); });
        });
        cable.register("close", function (_a) {
            var channel = _a.channel;
            var pipes = _this.channels.get(channel);
            if (pipes && pipes.source) {
                pipes.source.close();
            }
            return new Promise(function (resolve) { return setTimeout(resolve, 0); });
        });
        cable.register("connect", function (_a) {
            var uuid = _a.uuid, channel = _a.channel;
            try {
                var targetWs_1 = _this.router.get(uuid);
                var pipes_1 = { source: new Pipe_1.Pipe(dataPipe, channel, 32) };
                _this.channels.set(channel, pipes_1);
                pipes_1.source.onmessage = function (event) {
                    if (!_this.buffers.has(channel)) {
                        _this.buffers.set(channel, []);
                    }
                    _this.buffers.get(channel).push(event.data);
                };
                var connect_1 = function () {
                    if (!_this.clients.has(uuid)) {
                        var routedCablePipe = new Pipe_1.Pipe(targetWs_1, "WOHC");
                        var routedDataPipe = new Pipe_1.Pipe(targetWs_1, "WOHD");
                        _this.clients.set(uuid, { data: routedDataPipe, cable: routedCablePipe });
                    }
                    var client = _this.clients.get(uuid);
                    var targetCablePipe = client.cable;
                    var targetDataPipe = client.data;
                    var targetCable = new Cable_1.Cable(targetCablePipe);
                    pipes_1.target = new Pipe_1.Pipe(targetDataPipe, channel, 32);
                    pipes_1.target.onmessage = function (event) {
                        try {
                            pipes_1.source.send(event.data);
                        }
                        catch (e) {
                            // ignore
                        }
                    };
                    pipes_1.source.onclose = function (event) {
                        pipes_1.target.close();
                        try {
                            targetCable.notify("close", { channel: channel });
                        }
                        catch (e) {
                            // ignore
                        }
                        _this.channels.delete(channel);
                    };
                    targetCable.request("open", { channel: channel })
                        .then(function () {
                        var buffer = _this.buffers.get(channel) || [];
                        while (buffer.length) {
                            pipes_1.target.send(buffer.shift());
                        }
                        pipes_1.source.onmessage = function (event) {
                            pipes_1.target.send(event.data);
                        };
                    })
                        .catch(function () { return void 0; });
                };
                if (targetWs_1.readyState === WebSocket_1.default.OPEN) {
                    connect_1();
                }
                targetWs_1.addEventListener("open", function () {
                    connect_1();
                });
            }
            catch (e) {
                // ignore
            }
            return new Promise(function (resolve) { return setTimeout(resolve, 0); });
        });
        return function () {
            cable.destroy();
            cablePipe.close();
            dataPipe.close();
        };
    };
    return Server;
}());
exports.Server = Server;
//# sourceMappingURL=Server.js.map